@@ -1,10 +1,12 @@
 """IO connectors for Pub/Sub and Ray."""
 
+import asyncio
 import dataclasses
+from google.cloud import monitoring_v3
 import inspect
 import logging
 import json
-from typing import Any, Callable, Dict, Iterable, Union
+from typing import Any, Callable, Dict, Iterable, Optional, Union
 
 from google.cloud import pubsub
 from google.pubsub_v1.services.subscriber import SubscriberAsyncClient
@@ -14,6 +16,21 @@ from buildflow.api import io
 from buildflow.runtime.ray_io import base
 from buildflow.runtime.ray_io import pubsub_utils
 
+_BACKLOG_QUERY_TEMPLATE = """\
+fetch pubsub_subscription
+| metric 'pubsub.googleapis.com/subscription/num_unacked_messages_by_region'
+| filter
+    resource.project_id == '{project}'
+    && (resource.subscription_id == '{sub_id}')
+| group_by 1m,
+    [value_num_unacked_messages_by_region_mean:
+       mean(value.num_unacked_messages_by_region)]
+| every 1m
+| group_by [],
+    [value_num_unacked_messages_by_region_mean_aggregate:
+       aggregate(value_num_unacked_messages_by_region_mean)]
+"""
+
 
 @dataclasses.dataclass(frozen=True)
 class PubsubMessage:
@@ -22,7 +39,7 @@ class PubsubMessage:
 
 
 @dataclasses.dataclass
-class PubSubSource(io.Source):
+class PubSubSource(io.StreamingSource):
     """Source for connecting to a Pub/Sub subscription."""
 
     subscription: str
@@ -39,14 +56,27 @@ class PubSubSource(io.Source):
     def actor(self, ray_sinks):
         return PubSubSourceActor.remote(ray_sinks, self)
 
-    @classmethod
-    def is_streaming(cls) -> bool:
-        return True
+    def backlog(self) -> Optional[int]:
+        split_sub = self.subscription.split('/')
+        project = split_sub[1]
+        sub_id = split_sub[3]
+        client = monitoring_v3.QueryServiceClient()
+        result = client.query_time_series(
+            request=monitoring_v3.QueryTimeSeriesRequest(
+                name=f'projects/{project}',
+                query=_BACKLOG_QUERY_TEMPLATE.format(project=project,
+                                                     sub_id=sub_id),
+                page_size=1))
+        result_list = list(result)
+        if result_list:
+            # TODO: clean this up
+            return result_list[0].point_data[0].values[0].double_value
+        return None
 
     @classmethod
     def recommended_num_threads(cls):
         # The actor becomes mainly network bound after roughly 4 threads, and
-        # additoinal threads start to hurt cpu utilization.
+        # additional threads start to hurt cpu utilization.
         # This number is based on a single actor instance.
         return 4
 
@@ -64,8 +94,8 @@ class PubSubSink(io.Sink):
         return PubSubSinkActor.remote(remote_fn, self)
 
 
-@ray.remote
-class PubSubSourceActor(base.RaySource):
+@ray.remote(num_cpus=PubSubSource.num_cpus())
+class PubSubSourceActor(base.StreamingRaySource):
 
     def __init__(
         self,
@@ -83,8 +113,14 @@ class PubSubSourceActor(base.RaySource):
         while self.running:
             ack_ids = []
             payloads = []
-            response = await pubsub_client.pull(subscription=self.subscription,
-                                                max_messages=self.batch_size)
+            try:
+                response = await pubsub_client.pull(
+                    subscription=self.subscription,
+                    max_messages=self.batch_size,
+                    return_immediately=True)
+            except Exception as e:
+                logging.error('pubsub pull failed with: %s', e)
+                continue
             for received_message in response.received_messages:
                 json_loaded = {}
                 if received_message.message.data:
@@ -109,10 +145,21 @@ class PubSubSourceActor(base.RaySource):
                     await pubsub_client.acknowledge(
                         ack_ids=ack_ids, subscription=self.subscription)
                 except Exception as e:
-                    logging.error((
-                        'Failed to process message, '
-                        'will not be acked: error: %s'
-                    ), e)
+                    logging.error(('Failed to process message, '
+                                   'will not be acked: error: %s'), e)
+                    # This nacks the messages. See:
+                    # https://github.com/googleapis/python-pubsub/pull/123/files
+                    ack_deadline_seconds = 0
+                    pubsub_client.modify_ack_deadline(
+                        subscription=self.subscription,
+                        ack_ids=ack_ids,
+                        ack_deadline_seconds=ack_deadline_seconds)
+            else:
+                # This happens when we didn't get any messages.
+                await asyncio.sleep(3)
+            # For pub/sub we determine the utilization based on the number of
+            # messages received versus how many we received.
+            self.update_metrics(len(payloads))
 
     def shutdown(self):
         self.running = False
@@ -120,7 +167,8 @@ class PubSubSourceActor(base.RaySource):
         return True
 
 
-@ray.remote
+# TODO: put more though into this resource requirement
+@ray.remote(num_cpus=.25)
 class PubSubSinkActor(base.RaySink):
 
     def __init__(
