@@ -6,14 +6,16 @@ import logging
 import os
 import sys
 import traceback
-from typing import Dict, Iterable, Optional
+from typing import Dict, Optional
 
 import ray
 import requests
 
 from buildflow import utils
-from buildflow.api import ProcessorAPI, SourceType, SinkType
-from buildflow.runtime.ray_io import empty_io
+from buildflow.api import (FlowResults, ProcessorAPI, SourceType, SinkType,
+                           options)
+from buildflow.runtime.managers import batch_manager
+from buildflow.runtime.managers import stream_manager
 
 
 @dataclasses.dataclass
@@ -35,23 +37,40 @@ class Session:
     id: str
 
 
-@ray.remote
-class _ProcessActor(object):
+@dataclasses.dataclass
+class _StreamingResults(FlowResults):
 
-    def __init__(self, processor_instance: ProcessorAPI):
-        self._processor = processor_instance
-        print(f'Running processor setup: {self._processor.__class__}')
-        # NOTE: This is where the setup lifecycle method is called.
-        self._processor.setup()
+    def __init__(self, manager: stream_manager.StreamProcessManager) -> None:
+        self._manager = manager
 
-    def process(self, *args, **kwargs):
-        return self._processor._process(*args, **kwargs)
+    def output(self):
+        self._manager.block()
 
-    def process_batch(self, calls: Iterable):
-        to_ret = []
-        for call in calls:
-            to_ret.append(self.process(call))
-        return to_ret
+    def shutdown(self):
+        self._manager.shutdown()
+
+
+@dataclasses.dataclass
+class _BatchResults(FlowResults):
+
+    def __init__(self) -> None:
+        self._processor_tasks = {}
+
+    def _add_processor_task(self, processor_id: str, tasks):
+        self._processor_tasks[processor_id] = tasks
+
+    def output(self):
+        final_output = {}
+        for proc_id, batch_ref in self._processor_tasks.items():
+            proc_output = {}
+            output = ray.get(batch_ref)
+            for key, value in output.items():
+                if key in proc_output:
+                    proc_output[key].extend(value)
+                else:
+                    proc_output[key] = value
+            final_output[proc_id] = proc_output
+        return final_output
 
 
 def _load_session():
@@ -84,7 +103,7 @@ class Runtime:
         if args.disable_usage_stats:
             self._enable_usage = False
 
-    def run(self, num_replicas: int):
+    def run(self, streaming_options: options.StreamingOptions):
         if self._enable_usage:
             print(
                 'Usage stats collection is enabled. To disable add the flag: '
@@ -107,7 +126,7 @@ class Runtime:
         print('...Finished setting up resources')
 
         try:
-            output = self._run(num_replicas)
+            output = self._run(streaming_options)
             return output
         except Exception as e:
             print('Flow failed with error: ', e)
@@ -123,67 +142,24 @@ class Runtime:
         # TODO: Add support for multiple node types (i.e. endpoints).
         self._processors = {}
 
-    def _run(self, num_replicas):
-        running_tasks = {}
-        source_actors = {}
+    def _run(self, streaming_options: options.StreamingOptions):
+        batch_results = _BatchResults()
+        streaming_results = None
         for proc_id, processor_ref in self._processors.items():
-            key = str(processor_ref.sink)
-            if isinstance(processor_ref.sink, empty_io.EmptySink):
-                key = 'local'
-            source_actors[proc_id] = []
-            for i in range(num_replicas):
-                if i % 4 == 0:
-                    processor_actor = _ProcessActor.remote(
-                        processor_ref.get_processor_replica())
-                    sink_actor = processor_ref.sink.actor(
-                        processor_actor.process_batch.remote,
-                        processor_ref.source.is_streaming())
+            if not processor_ref.source.is_streaming():
+                manager = batch_manager.BatchProcessManager(processor_ref)
+                batch_results._add_processor_task(proc_id, manager.run())
+            else:
+                assert len(self._processors) == 1
+                manager = stream_manager.StreamProcessManager(
+                    processor_ref, streaming_options)
+                manager.run()
+                streaming_results = _StreamingResults(manager)
 
-                source_actor = processor_ref.source.actor({key: sink_actor})
-                source_actors[proc_id].append(source_actor)
-                num_threads = processor_ref.source.recommended_num_threads()
-                source_pool_tasks = [
-                    source_actor.run.remote() for _ in range(num_threads)
-                ]
+        if streaming_results is not None:
+            return streaming_results
 
-            # We no longer need to use the Actor Pool because there's no input
-            # to the actors (they spawn their own inputs based on the IO refs).
-            # We also need to await each actor's subtask separately because its
-            # now running on multiple threads.
-            running_tasks[proc_id] = source_pool_tasks
-
-        final_output = {}
-        for proc_id, tasks in running_tasks.items():
-            try:
-                all_actor_outputs = ray.get(tasks)
-            except KeyboardInterrupt:
-                print('Shutting down processors...')
-                sources = source_actors[proc_id]
-                any_block = False
-                for source in sources:
-                    should_block = ray.get(source.shutdown.remote())
-                    if should_block:
-                        any_block = True
-                # If any of the shutdown operations require blocking wait.
-                # In practice if one requires blocking they all should.
-                if any_block:
-                    ray.get(tasks)
-                print('...Sucessfully shut down processors.')
-                return
-
-            # TODO: Add option to turn this off for prod deployments
-            # Otherwise I think we lose time to sending extra data over the
-            # wire.
-            processor_output = {}
-            for actor_output in all_actor_outputs:
-                if actor_output is not None:
-                    for key, value in actor_output.items():
-                        if key in processor_output:
-                            processor_output[key].extend(value)
-                        else:
-                            processor_output[key] = value
-            final_output[proc_id] = processor_output
-        return final_output
+        return batch_results
 
     def register_processor(self,
                            processor_instance: ProcessorAPI,
@@ -194,6 +170,20 @@ class Runtime:
             logging.warning(
                 f'Processor {processor_id} already registered. Overwriting.')
 
+        # For a flow allow 1 streaming processors or N batch processors.
+        # If user is adding a streaming processor ensure their are no other
+        # processors.
+        # If user is adding a batch processor ensure their are no existing
+        # streaming processors.
+        if ((processor_instance.source().is_streaming() and self._processors)
+                or any([
+                    p.source.is_streaming()
+                    for p in self._processors.values()
+                ])):
+            raise ValueError(
+                'Flows containing a streaming processor are only allowed '
+                'to have one processor.')
+
         # NOTE: This is where the source / sink lifecycle methods are executed.
         self._processors[processor_id] = _ProcessorRef(
             processor_instance, processor_instance.source(),
